/************************************************************************/
/*																		*/
/*		Name:		IO_DOS.C                                            */
/*		Function:	SDL Hardware Interface					            */
/*		Author:		Paul Robson      									*/
/*		Created:	2nd December 2001                                   */
/*		Modified:	15th February 2003                                  */
/*		Project:	The Valley in C										*/
/*																		*/
/************************************************************************/

#define DRIVER
#include "valley.h"
#include "SDL.h"

static Uint32 _HWSpecToSDL(int Colour);

static int ToHW[16] = { 0,12,10,14,9,13,11,15,2,7,7,7,7,7,7,7 };

static SDL_Surface *Display = NULL;

/************************************************************************/
/*                                                                      */
/*              Convert Speccy Colour to SDL Colour format              */
/*                                                                      */
/************************************************************************/

static Uint32 _HWSpecToSDL(int Colour)
{
    int r,g,b,l;
    Colour = ToHW[Colour & 15];                   /* Convert colour to hw colour */
    l = (Colour & 0x8) ? 255:128;
    r = (Colour & 4) ? l:0;
    g = (Colour & 2) ? l:0;
    b = (Colour & 1) ? l:0;
    return SDL_MapRGB(Display->format,r,g,b);
}

/************************************************************************/
/*						Initialise the Hardware I/O						*/
/************************************************************************/

void HWInitialise(void)
{
    char Msg[128];
    if (SDL_Init(SDL_INIT_VIDEO) < 0) exit(-1);     /* Try to initialise SDL */
    atexit(SDL_Quit);
    Display = SDL_SetVideoMode(1024,768,16,SDL_SWSURFACE|SDL_ANYFORMAT);
    if (Display == NULL) VERROR();
    SDL_EnableUNICODE(1);
    sprintf(Msg,"The Valley %s",VERSION);
    SDL_WM_SetCaption(Msg,NULL);
   
}


/************************************************************************/
/*						   Terminate Hardware I/O						*/
/************************************************************************/

void HWTerminate(void)
{
    SDL_Quit();
}


/************************************************************************/
/*			Read key, return ASCII code or 0 if no key pressed			*/
/************************************************************************/

int HWInkey(void)
{
    SDL_Event Event;
    int ch;
    SDL_UpdateRect(Display,0,0,Display->w,Display->h);
    if (SDL_PollEvent(&Event) == 0) return 0;
    if (Event.type != SDL_KEYDOWN) return 0;
    ch = Event.key.keysym.unicode;
    if (ch & 0xFF80) return 0;
    ch = ch & 0x7F;ch = toupper(ch);
    return ch;
}


/************************************************************************/
/*					Return elapsed time in milliseconds					*/
/************************************************************************/

long HWGetTimer(void)
{
    SDL_UpdateRect(Display,0,0,Display->w,Display->h);
    return SDL_GetTicks();
}


/************************************************************************/
/*							Print a character							*/
/************************************************************************/

void HWPrint(int x,int y,int Colour,int Char)
{
    SDL_Rect rc;
    Uint32 Col;
    if (x < 0 || y < 0) return;                   /* Check range */
    if (x >= SCX || y >= SCY) return;

    rc.w = Display->w/SCX;
    rc.h = Display->h/SCY;

    rc.w &= 0xFFFC;
    rc.h &= 0xFFFC;

    rc.x = x * rc.w + (Display->w-rc.w*SCX)/2;
    rc.y = y * rc.h + (Display->h-rc.h*SCY)/2;
    SDL_FillRect(Display,&rc,SDL_MapRGB(Display->format,0,0,0));
    if (Char == 32) return;                       /* Space */
    if (Char > 32 && Char < 126)                  /* If it's a character */
    {
        Col = _HWSpecToSDL(Colour);
        FONTChar(Display,&rc,Col,Char);
    }
    if (Char >= 224)                              /* If it's a character */
    {
        Col = _HWSpecToSDL(Colour);
        FONTChar(Display,&rc,Col,Char-224+128);
    }
}


/************************************************************************/
/*								Report an Error							*/
/************************************************************************/

void HWError(int Line,char *File)
{
    HWTerminate();
    fprintf(stderr,"Error at %d of %s.\n\n",Line,File);
    exit(1);
}

